@@ -261,17 +261,15 @@ check.nthreads <- function(nthreads) {
         nthreads <- 1L
     } else if (is.na(nthreads)) {
         nthreads <- 1L
-    } else if (nthreads == "auto") {
-        nthreads <- parallel::detectCores()
     } else if (nthreads < 1L) {
-        nthreads <- parallel::detectCores()
+        nthreads <- 1L
     }
     nthreads <- as.integer(nthreads)
     if (nthreads > 1L && !R_has_openmp()) {
         msg <- paste0("Attempting to use more than 1 thread, but ",
                       "package was compiled without OpenMP support.")
         if (tolower(Sys.info()[["sysname"]]) == "darwin")
-            msg <- paste0(msg, " See https://mac.r-project.org/openmp/")
+            msg <- paste0(msg, " See https://github.com/david-cortes/installing-optimized-libraries#4-macos-install-and-enable-openmp")
         warning(msg)
     }
     return(nthreads)
@@ -292,7 +290,9 @@ check.is.model.obj <- function(model_obj) {
     if (!("outliertree" %in% class(model_obj))) {
         stop("Must pass an Outlier Tree model object as generated by function 'outlier.tree'.")
     }
-    if (is.null(model_obj$obj_from_cpp)) {
+    if (is.null(model_obj$obj_from_cpp$ptr_model$ptr) ||
+        identical(model_obj$obj_from_cpp$ptr_model$ptr, methods::new("externalptr"))
+    ) {
         stop("Outlier Tree model object has been corrupted.")
     }
 }
